import requests
from mlops.config.config import settings
from mlops.src.mailgun import MailGun
from mlops.exceptions.tracking import request_failed_exception
from mlops.exceptions.monitoring import failed_to_set_active_model_exception
import pandas as pd


def get_model_by_name(model_name: str) -> dict:
    """
    Function for retrieving mlops monitored model from database

    Args:
        model_name: unique name of the monitored model to be retrieved

    Returns:
        monitored_model: json data of monitored model
    """
    app_response = requests.get(f"{settings.url}/monitored-models/name/{model_name}")
    response_json = app_response.json()

    if app_response.status_code == 200:
        return response_json
    else:
        raise request_failed_exception(app_response)


def create_model(model_name: str, model_description: str = None, iteration_dict: dict = None) -> dict:
    """
    Function for creating mlops monitored model

    Args:
        model_name: unique name of the created name
        model_description: description of monitored model
        iteration_dict: dictionary containing valid iteration data with a path to model

    Returns:
        monitored_model: json data of monitored model
    """
    if iteration_dict is None:
        model_status = "idle"
    else:
        model_status = "active"

    data = {
        "model_name": model_name,
        "model_description": model_description,
        "model_status": model_status,
        "iteration": iteration_dict
    }

    app_response = requests.post(f"{settings.url}/monitored-models/", json=data)
    response_json = app_response.json()

    if app_response.status_code == 201:
        return response_json
    else:
        raise request_failed_exception(app_response)


def set_active_model(model_name: str) -> str:
    """
    Function for setting active model from monitored models

    Args:
        model_name: Name of monitored model, that will be set as active

    Returns:
        Information about new active model setup
    """
    try:
        model = get_model_by_name(model_name)
    except Exception as e:
        raise failed_to_set_active_model_exception(e)

    settings.change_active_model(model["model_name"])
    return f"Active model set to: {settings.active_model}"


def send_prediction(model_name: str, data: pd.DataFrame, send_email: bool = False) -> dict:
    """
    Function to invoke a prediction from monitored model. Function takes a pandas dataframe, where every record is
    taken as a separate prediction.

    Args:
        send_email: Email alert flag
        model_name: Name of monitored model that will be used in prediction
        data: Pandas Dataframe containing data for prediction

    Returns:
        List of dictionaries containing results for each executed prediction
    """
    model = get_model_by_name(model_name)

    if send_email or settings.send_emails:
        mailgun = MailGun()

    data_json = data.to_dict(orient="records")

    app_response = requests.post(f"{settings.url}/monitored-models/{model['_id']}/predict", json=data_json)

    prediction = app_response.json()

    if app_response.status_code == 200:
        if send_email or settings.send_emails:
            mailgun.send_prediction_success(prediction)
        return prediction
    else:
        if send_email or settings.send_emails:
            mailgun.send_prediction_failure(request_failed_exception(app_response))
        raise request_failed_exception(app_response)



from requests import Response


def iteration_request_failed_exception(response: Response):
    detail = response.json()['detail']
    raise Exception(f"Iteration not created. Request failed with status code {response.status_code}: {detail}")


def model_path_not_exist_exception():
    raise FileNotFoundError("Provided model path does not exist.")


def monitored_model_encoding_pkl_file_exception(description: str):
    raise Exception(f"Cannot encode pkl file: {description}")


from requests import Response


def project_id_is_none_exception():
    return ValueError('Project ID not specified')


def experiment_id_is_none_exception():
    return ValueError('Experiment ID not specified')


def failed_to_set_active_project_exception(e):
    return Exception(f"Failed to set active project: {e}")


def failed_to_set_active_experiment_exception(e):
    return Exception(f"Failed to set active experiment: {e}")


def request_failed_exception(response: Response):
    detail = response.json()['detail']
    return Exception(f"Request failed with status code {response.status_code}: {detail}")



from mlops.config.config import settings
from mlops.exceptions.mailgun import mail_request_failed_exception

from json2html import json2html
import requests
import json


class MailGun:
    """
    Class for sending emails using MailGun API.
    """
    def __init__(
            self
    ):
        self.api_key = settings.mailgun_api_key
        self.domain = settings.mailgun_domain
        self.user_email = settings.user_email

    def _validate_variables(self):
        """
        Utility function for validating variables.
        """
        if self.api_key is None:
            raise ValueError("MailGun API key is not set.")
        if self.domain is None:
            raise ValueError("MailGun domain is not set.")
        if self.user_email is None:
            raise ValueError("User email is not set.")

    def send_tracking_success(self, iteration_response: dict) -> requests.Response:
        """
        Function for sending success tracking (iteration) email.

        Args:
            iteration_response: response from iteration request

        Returns:
            response: response from MailGun API
        """
        self._validate_variables()

        # TODO: substitute with a proper html template later
        html_content = f"""
                    <!DOCTYPE html>
                    <html lang="en">
                    <head>
                        <meta charset="UTF-8">
                        <meta name="viewport" content="width=device-width, initial-scale=1">
                        <style>
                            body {{
                                font-family: 'Arial', sans-serif;
                                background-color: #f4f4f4;
                                text-align: center;
                                padding: 20px;
                            }}
                            h1 {{
                                color: #2ecc71; /* Green color for success */
                            }}
                            p {{
                                color: #555;
                                font-size: 16px;
                                margin-top: 10px;
                            }}
                            .success-icon {{
                                color: #2ecc71; /* Green color for success */
                                font-size: 50px;
                            }}
                        </style>
                    </head>
                    <body>
                        <div class="success-icon">&#10003;</div>
                        <h1>Iteration Created Successfully!</h1>
                        <p>Your iteration has been successfully created. Here are some details:</p>
                        <div>{json2html.convert(json=json.dumps(iteration_response))}</div>
                    </body>
                    </html>
                    """

        response = requests.post(
            f"https://api.mailgun.net/v3/{self.domain}/messages",
            auth=("api", f"{self.api_key}"),
            data={"from": f"MLOps mailgun <mailgun@{self.domain}>",
                  "to": [f"{self.user_email}"],
                  "subject": "MLOps iteration created successfully",
                  "html": html_content}
        )

        if response.status_code == 200:
            return response
        else:
            raise mail_request_failed_exception(response)

    def send_tracking_failure(self, iteration_exception_response: str) -> requests.Response:
        """
        Function for sending failure tracking (iteration) email.

        Args:
            iteration_exception_response: exception response from iteration request

        Returns:
            response: response from MailGun API
        """
        self._validate_variables()

        # TODO: substitute with a proper html template later
        html_content = f"""
                    <!DOCTYPE html>
                    <html lang="en">
                    <head>
                        <meta charset="UTF-8">
                        <meta name="viewport" content="width=device-width, initial-scale=1">
                        <style>
                            body {{
                                font-family: 'Arial', sans-serif;
                                background-color: #f4f4f4;
                                text-align: center;
                                padding: 20px;
                            }}
                            h1 {{
                                color: #e74c3c; /* Red color for failure */
                            }}
                            p {{
                                color: #555;
                                font-size: 16px;
                                margin-top: 10px;
                            }}
                            .failure-icon {{
                                color: #e74c3c; /* Red color for failure */
                                font-size: 50px;
                            }}
                            .exception-details {{
                                text-align: left;
                                margin: 10px;
                                padding: 10px;
                                border: 1px solid #ddd;
                                background-color: #fff;
                                overflow: auto;
                            }}
                        </style>
                    </head>
                    <body>
                        <div class="failure-icon">&#10008;</div>
                        <h1>Iteration Failed!</h1>
                        <p>An exception occurred during the iteration. Here are the details:</p>
                        <div class="exception-details">{iteration_exception_response}</div>
                    </body>
                    </html>
                    """

        response = requests.post(
            f"https://api.mailgun.net/v3/{self.domain}/messages",
            auth=("api", f"{self.api_key}"),
            data={"from": f"MLOps mailgun <mailgun@{self.domain}>",
                  "to": [f"{self.user_email}"],
                  "subject": "MLOps iteration failed",
                  "html": html_content}
        )

        if response.status_code == 200:
            return response
        else:
            raise mail_request_failed_exception(response)

    def send_prediction_success(self, prediction_data: dict) -> requests.Response:
        """
        Function for sending prediction success email.

        Args:
            prediction_data: response from send_prediction request

        Returns:
            response: response from MailGun API
        """
        self._validate_variables()

        # TODO: substitute with a proper html template later
        html_content = f"""
                    <!DOCTYPE html>
                    <html lang="en">
                    <head>
                        <meta charset="UTF-8">
                        <meta name="viewport" content="width=device-width, initial-scale=1">
                        <style>
                            body {{
                                font-family: 'Arial', sans-serif;
                                background-color: #f4f4f4;
                                text-align: center;
                                padding: 20px;
                            }}
                            h1 {{
                                color: #2ecc71; /* Green color for success */
                            }}
                            p {{
                                color: #555;
                                font-size: 16px;
                                margin-top: 10px;
                            }}
                            .success-icon {{
                                color: #2ecc71; /* Green color for success */
                                font-size: 50px;
                            }}
                        </style>
                    </head>
                    <body>
                        <div class="success-icon">&#10003;</div>
                        <h1>Prediction Sent Successfully!</h1>
                        <p>Your prediction has been sent successfully. Here are some details:</p>
                        <div>{json2html.convert(json=json.dumps(prediction_data))}</div>
                    </body>
                    </html>
                    """

        response = requests.post(
            f"https://api.mailgun.net/v3/{self.domain}/messages",
            auth=("api", f"{self.api_key}"),
            data={"from": f"MLOps mailgun <mailgun@{self.domain}>",
                  "to": [f"{self.user_email}"],
                  "subject": "MLOps prediction sent successfully",
                  "html": html_content}
        )

        if response.status_code == 200:
            return response
        else:
            raise mail_request_failed_exception(response)

    def send_prediction_failure(self, prediction_exception_response: str) -> requests.Response:
        """
        Function for sending prediction failure email.

        Args:
            prediction_exception_response: exception response from iteration request

        Returns:
            response: response from MailGun API
        """
        self._validate_variables()

        # TODO: substitute with a proper html template later
        html_content = f"""
                    <!DOCTYPE html>
                    <html lang="en">
                    <head>
                        <meta charset="UTF-8">
                        <meta name="viewport" content="width=device-width, initial-scale=1">
                        <style>
                            body {{
                                font-family: 'Arial', sans-serif;
                                background-color: #f4f4f4;
                                text-align: center;
                                padding: 20px;
                            }}
                            h1 {{
                                color: #e74c3c; /* Red color for failure */
                            }}
                            p {{
                                color: #555;
                                font-size: 16px;
                                margin-top: 10px;
                            }}
                            .failure-icon {{
                                color: #e74c3c; /* Red color for failure */
                                font-size: 50px;
                            }}
                            .exception-details {{
                                text-align: left;
                                margin: 10px;
                                padding: 10px;
                                border: 1px solid #ddd;
                                background-color: #fff;
                                overflow: auto;
                            }}
                        </style>
                    </head>
                    <body>
                        <div class="failure-icon">&#10008;</div>
                        <h1>Prediction Failed!</h1>
                        <p>An exception occurred during the prediction. Here are the details:</p>
                        <div class="exception-details">{prediction_exception_response}</div>
                    </body>
                    </html>
                    """

        response = requests.post(
            f"https://api.mailgun.net/v3/{self.domain}/messages",
            auth=("api", f"{self.api_key}"),
            data={"from": f"MLOps mailgun <mailgun@{self.domain}>",
                  "to": [f"{self.user_email}"],
                  "subject": "MLOps prediction failed",
                  "html": html_content}
        )

        if response.status_code == 200:
            return response
        else:
            raise mail_request_failed_exception(response)




import os
import base64
import pickle

import requests

from mlops.config.config import settings
from mlops.src.chart import Chart
from mlops.src.mailgun import MailGun
from mlops.exceptions.tracking import request_failed_exception
from mlops.exceptions.iteration import (
    iteration_request_failed_exception,
    model_path_not_exist_exception, monitored_model_encoding_pkl_file_exception
)


class Iteration:
    """
    Class for logging iteration data.
    """

    def __init__(
            self, iteration_name: str,
            project_id: str = None,
            experiment_id: str = None,
            send_email: bool = False
    ):
        self.iteration_name: str = iteration_name
        self.project_id: str = project_id
        self.experiment_id: str = experiment_id
        self.user_name: str = settings.user_name
        self.send_email: bool = send_email
        self.path_to_model: str = ''
        self.encoded_ml_model: str = None
        self.parameters: dict = {}
        self.metrics: dict = {}
        self.dataset_id: str = None
        self.charts: list = []
        self.dataset_name: str = None
        self.has_dataset: bool = False
        self.image_charts: list = []

    def format_path(self):
        self.path_to_model = self.path_to_model.replace('\f', '\\f').replace('\t', '\\t').replace(
            '\n', '\\n').replace('\r', '\\r').replace('\b', '\\b').replace('/', '\\')

    def path_to_model_exists(self) -> bool:
        """
        Checking if path to model exists.

        Returns:
            True if path to model exists, Exception otherwise.
        """
        if os.path.exists(self.path_to_model):
            return True
        else:
            raise model_path_not_exist_exception()

    def log_path_to_model(self, path_to_model: str):
        """
        Logging path to model.

        Args:
            path_to_model: input path to model
        """
        self.path_to_model = path_to_model
        self.format_path()
        self.path_to_model_exists()

        try:
            # Load the model from the file
            _, file_extension = os.path.splitext(self.path_to_model)

            if file_extension in ['.pkl', '.pickle']:
                self.encoded_ml_model = self.__encode_ml_model(self.path_to_model)
            else:
                raise monitored_model_encoding_pkl_file_exception("It is not a pickle file.")

        except Exception as e:
            # Handle any exceptions or errors that may occur
            raise monitored_model_encoding_pkl_file_exception(str(e))



    def log_metric(self, metric_name: str, value):
        """
        Logging single metric.

        Args:
            metric_name: name of the logged metric
            value: value of the logged metric
        """
        self.metrics[metric_name] = value

    def log_metrics(self, metrics: dict):
        """
        Logging multiple metrics.

        Args:
            metrics: dictionary of metrics
        """
        self.metrics.update(metrics)

    def log_parameter(self, parameter_name: str, value):
        """
        Logging single parameter.

        Args:
            parameter_name: name of the logged parameter
            value: value of the logged parameter
        """
        self.parameters[parameter_name] = value

    def log_parameters(self, parameters: dict):
        """
        Logging multiple parameters.

        Args:
            parameters: dictionary of parameters
        """
        self.parameters.update(parameters)

    def log_dataset(self, dataset_id: str):
        """
        Logging dataset

        Args:
            dataset_id: string containing dataset id
        """

        app_response = requests.get(f"{settings.url}/datasets/{dataset_id}")

        response_json = app_response.json()

        if app_response.status_code == 200:
            self.dataset_name = response_json["dataset_name"]
            self.dataset_id = dataset_id
            self.has_dataset = True
        else:
            raise request_failed_exception(app_response)

    def log_chart(self, chart_name: str, chart_type: str, chart_title: str, chart_subtitle: str = None,
                  x_data: list = [list],
                  y_data: list = [list], y_data_names: [str] = [], x_label: str = "x", y_label: str = "y",
                  x_min: float = None, x_max: float = None, y_min: float = None, y_max: float = None,
                  comparable: bool = False):
        """
        Logging a single chart

        Args:
            **chart_name (str)**: Chart name.
            **chart_type (str)**: Chart type.
            **x_data(List[float])**: X data.
            **y_data (List[float])**: Y data.
            **y_data_names (List[str])**: List of y data names
            **x_label (Optional[str])**: X label.
            **y_label (Optional[str])**: Y label.
            **x_min (Optional float)**: Minimal value of x.
            **y_min (Optional float)**: Minimal value of y.
            **x_max (Optional float)**: Maximum value of x.
            **y_max (Optional float)**: Maximum value of y.
            **comparable (Optional bool)**: Determines whether chart can be compared with other charts
        """

        chart = Chart(chart_name=chart_name, chart_type=chart_type, chart_title=chart_title,
                      chart_subtitle=chart_subtitle, x_data=x_data,
                      y_data=y_data, y_data_names=y_data_names, x_label=x_label, y_label=y_label,
                      x_min=x_min, x_max=x_max, y_min=y_min, y_max=y_max, comparable=comparable)

        self.charts.append(chart)

    def transform_charts_to_dictionary(self):
        """

        Returns:

        """

        interactive_charts = []

        for chart in self.charts:
            interactive_charts.append(chart.get_chart_dictionary())

        return interactive_charts

    def log_image_chart(self, name: str, image_path: str):
        """
        Logging image chart

        Args:
            image_path: path to the image chart
            name: name of the image chart
        """
        with open(image_path, "rb") as image_file:
            encoded_image = base64.b64encode(image_file.read()).decode('utf-8')

        self.image_charts.append({"name": name, "encoded_image": encoded_image})

    def end_iteration(self) -> dict or None:
        """
        End iteration and send data to API.

        Returns:
            iteration: json data of created iteration
        """
        if self.dataset_id:
            dataset = {"id": self.dataset_id}
        else:
            dataset = None

        if self.charts:
            interactive_charts = self.transform_charts_to_dictionary()
        else:
            interactive_charts = None

        data = {
            "user_name": self.user_name,
            "iteration_name": self.iteration_name,
            "metrics": self.metrics,
            "parameters": self.parameters,
            "path_to_model": self.path_to_model,
            "encoded_ml_model": self.encoded_ml_model,
            "dataset": dataset,
            "image_charts": self.image_charts,
            "interactive_charts": interactive_charts
        }

        app_response = requests.post(
            f'{settings.url}/projects/{self.project_id}/experiments/{self.experiment_id}/iterations/', json=data)

        response_json = app_response.json()

        if app_response.status_code == 201:
            return response_json
        else:
            # this is a bit confusing, but mailgun for exception needs to be invoked twice
            # one is for the Iteration class exceptions itself, it is invoked inside tracking start_iteration() function
            # and the other one here is for the exceptions after sending the request to the API
            mailgun = MailGun()
            detail = response_json['detail']
            if self.send_email or settings.send_emails:
                mailgun.send_tracking_failure(
                    f"Request failed with status code {app_response.status_code}: {detail}"
                )

            raise iteration_request_failed_exception(app_response)

    @staticmethod
    def __encode_ml_model(path_to_model: str):
        """
        Encode ml model.

        Args:
            path_to_model: Path to model.
        """

        try:
            # Load the model from the file
            _, file_extension = os.path.splitext(path_to_model)

            if file_extension in ['.pkl', '.pickle']:

                ml_model = pickle.load(open(path_to_model, 'rb'))

                # Serialize the model
                ml_model = pickle.dumps(ml_model)

                # Encode the serialized model as base64
                ml_model = base64.b64encode(ml_model).decode("utf-8")

                return ml_model
            else:
                raise monitored_model_encoding_pkl_file_exception("It is not a pickle file.")

        except Exception as e:
            # Handle any exceptions or errors that may occur
            raise monitored_model_encoding_pkl_file_exception(str(e))



import requests
from mlops.config.config import settings
from mlops.exceptions.tracking import request_failed_exception


class Dataset:
    """
    Class for logging datasets
    """

    def __init__(self, dataset_name: str, path_to_dataset: str, dataset_description: str = None,
                 tags: str = None, version: str = None):
        self.dataset_name: str = dataset_name
        self.path_to_dataset: str = path_to_dataset
        self.dataset_description: str = dataset_description
        self.tags: str = tags
        self.version: str = version

    def get_dataset_json(self) -> dict:
        """
        Transform dataset values into dictionary to be used in http request body
        Returns:
            dataset_dict: dictionary containing dataset parameters
        """
        dataset_dict = {
            "dataset_name": self.dataset_name,
            "path_to_dataset": self.path_to_dataset,
            "dataset_description": self.dataset_description,
            "tags": self.tags,
            "version": self.version
        }

        return dataset_dict

    def create_dataset_in_app(self) -> dict:
        """
        Create dataset in webapp.

        Returns:
            dataset: json data of the created dataset
        """
        data = self.get_dataset_json()

        app_response = requests.post(f"{settings.url}/datasets/", json=data)

        response_json = app_response.json()

        if app_response.status_code == 201:
            return response_json
        else:
            raise request_failed_exception(app_response)



import requests
from contextlib import contextmanager
from mlops.config.config import settings
from mlops.src.iteration import Iteration
from mlops.src.dataset import Dataset
from mlops.src.mailgun import MailGun
from mlops.exceptions.tracking import project_id_is_none_exception, experiment_id_is_none_exception, \
    failed_to_set_active_project_exception, failed_to_set_active_experiment_exception, request_failed_exception
from typing import ContextManager


def get_project(project_id: str = None) -> dict:
    """
    Function for getting projects from mlops server

    Args:
        project_id: Id od the desired project, that will be retrieved from mlops app

    Returns:
        project: json data of the project
    """
    project_id = settings.active_project_id if not project_id else project_id

    if project_id is None:
        raise project_id_is_none_exception()

    app_response = requests.get(f"{settings.url}/projects/{project_id}")
    response_json = app_response.json()

    if app_response.status_code == 200:
        return response_json
    else:
        raise request_failed_exception(app_response)


def get_project_by_name(project_title: str) -> dict:
    """
    Function for getting projects from mlops server by name

    Args:
        project_title: name od the desired project, that will be retrieved from mlops app

    Returns:
        project: json data of the project
    """
    app_response = requests.get(f"{settings.url}/projects/title/{project_title}")
    response_json = app_response.json()

    if app_response.status_code == 200:
        return response_json
    else:
        raise request_failed_exception(app_response)


def create_project(title: str, description: str = None,
                   status: str = 'not_started', archived: bool = False) -> dict:
    """
    Function for creating mlops projects

    Args:
        title: Title of the created project
        description: Description of the created project (optional)
        status: Status of the created project (optional)
        archived: Archived status of the created project (optional)

    Returns:
        project: JSON data of the created project
    """
    data = {
        "title": title,
        "description": description,
        "status": status,
        "archived": archived
    }

    app_response = requests.post(f"{settings.url}/projects/", json=data)
    response_json = app_response.json()

    if app_response.status_code == 201:
        return response_json
    else:
        raise request_failed_exception(app_response)


def set_active_project(project_id: str) -> str:
    """
    Function for setting active project

    Args:
        project_id: Id of the project, that will be set as active

    Returns:
        project: JSON data of the active project
    """
    try:
        project = get_project(project_id)
    except Exception as e:
        raise failed_to_set_active_project_exception(e)

    settings.change_active_project(project_id)

    return f"Active project set to: {settings.active_project_id}"


def get_experiment(experiment_id: str = None, project_id: str = None) -> dict:
    """
    Function for getting projects from mlops server

    Args:
        experiment_id: Id of the experiment, that will be retrieved from mlops app
        project_id: Id of the project, that the experiment comes from (optional)

    Returns:
        experiment: json data of the experiment
    """
    experiment_id = settings.active_experiment_id if not experiment_id else experiment_id
    project_id = settings.active_project_id if not project_id else project_id

    if project_id is None:
        raise project_id_is_none_exception()
    if experiment_id is None:
        raise experiment_id_is_none_exception()

    app_response = requests.get(f"{settings.url}/projects/{project_id}/experiments/{experiment_id}")

    if app_response.status_code == 200:
        experiment = app_response.json()
        return experiment
    else:
        raise request_failed_exception(app_response)


def get_experiment_by_name(experiment_name: str, project_id: str = None) -> dict:
    """
    Function for getting projects from mlops server by name

    Args:
        experiment_name: Name of the experiment, that will be retrieved from mlops app
        project_id: Id of the project, that the experiment comes from (optional)

    Returns:
        experiment: json data of the experiment
    """
    project_id = settings.active_project_id if not project_id else project_id

    if project_id is None:
        raise project_id_is_none_exception()

    app_response = requests.get(f"{settings.url}/projects/{project_id}/experiments/name/{experiment_name}")

    if app_response.status_code == 200:
        experiment = app_response.json()
        return experiment
    else:
        raise request_failed_exception(app_response)


def create_experiment(name: str, description: str = None,
                      project_id: str = None) -> dict:
    """
    Function for creating mlops experiments

    Args:
        name: Name of the created experiment
        description: Description of the created experiment (optional)
        project_id: Id of the project, that the experiment comes from (optional)

    Returns:
        experiment: json data of the created experiment
    """
    project_id = settings.active_project_id if not project_id else project_id

    if project_id is None:
        raise project_id_is_none_exception()

    data = {
        "name": name,
        "description": description,
    }

    app_response = requests.post(f"{settings.url}/projects/{project_id}/experiments/", json=data)
    response_json = app_response.json()

    if app_response.status_code == 201:
        return response_json
    else:
        raise request_failed_exception(app_response)


def set_active_experiment(experiment_id: str) -> str:
    """
    Function for setting active experiment

    Args:
        experiment_id: Id of the experiment, that will be set as active

    Returns:
        experiment: json data of the active experiment
    """
    try:
        experiment = get_experiment(experiment_id, settings.active_project_id)
    except Exception as e:
        raise failed_to_set_active_experiment_exception(e)

    settings.change_active_experiment(experiment_id)
    return f"Active experiment set to: {settings.active_experiment_id}"


def create_dataset(dataset_name: str, path_to_dataset: str, dataset_description: str = None,
                   tags: str = '', version: str = None) -> dict:
    """
    Function for creating mlops datasets

    Args:
        dataset_name: name of the created dataset
        path_to_dataset: path to dataset files
        dataset_description: short description of the dataset displayed in the app
        tags: tags for dataset
        version: version of the dataset

    Returns:
        dataset: json data of created dataset
    """

    dataset = Dataset(dataset_name, path_to_dataset, dataset_description, tags, version)

    app_response = dataset.create_dataset_in_app()

    return app_response


def get_dataset(dataset_name: str, dataset_version: str):
    """
    Function for getting mlops datasets based on name and version

    Args:
        dataset_name: name of the dataset
        dataset_version: version of the dataset

    Returns:
        dataset: json data of the dataset
    """
    app_response = requests.get(f"{settings.url}/datasets/name/{dataset_name}/version/{dataset_version}")

    if app_response.status_code == 200:
        dataset = app_response.json()
        return dataset
    else:
        raise request_failed_exception(app_response)


@contextmanager
def start_iteration(iteration_name: str, project_id: str = None,
                    experiment_id: str = None, send_email: bool = False) -> ContextManager[Iteration]:
    """
    Function for creating mlops iteration

    Args:
        iteration_name: name of the created iteration
        project_id: if passed id of the project, else active project_id from settings
        experiment_id: if passed id of the experiment, else active experiment_id from settings
        send_email: if True, email will be sent after iteration ends

    Returns:
        Iteration.end_iteration() method output
    """
    project_id = settings.active_project_id if not project_id else project_id
    experiment_id = settings.active_experiment_id if not experiment_id else experiment_id

    if project_id is None:
        raise project_id_is_none_exception()
    if experiment_id is None:
        raise experiment_id_is_none_exception()

    iteration = Iteration(
        iteration_name=iteration_name,
        project_id=project_id,
        experiment_id=experiment_id,
        send_email=send_email
    )
    mailgun = MailGun()
    exception_occurred = False

    try:
        yield iteration
    except Exception as e:
        exception_occurred = True
        if send_email or settings.send_emails:
            mailgun.send_tracking_failure(str(e))
        raise e
    finally:
        if not exception_occurred:
            output = iteration.end_iteration()
            if send_email or settings.send_emails:
                mailgun.send_tracking_success(output)





class Chart:
    """
    Class for handling plots inside library
    """
    def __init__(self, chart_name: str, chart_type: str, chart_title: str, chart_subtitle: str = None,  x_data: list = [list],
                 y_data: list = [list], y_data_names: [str] = [], x_label: str = "x", y_label: str = "y",
                 x_min: float = None, x_max: float = None, y_min: float = None, y_max: float = None, comparable: bool = False):
        """
        Interactive chart model.

        Attributes:
        - **id (PydanticObjectId)**: Chart id.
        - **chart_name (str)**: Chart name.
        - **chart_type (str)**: Chart type.
        - **x_data(List[float])**: X data.
        - **y_data (List[float])**: Y data.
        - **y_data_names (List[str])**: List of y data names
        - **x_label (Optional[str])**: X label.
        - **y_label (Optional[str])**: Y label.
        - **x_min (Optional float)**: Minimal value of x.
        - **y_min (Optional float)**: Minimal value of y.
        - **x_max (Optional float)**: Maximum value of x.
        - **y_max (Optional float)**: Maximum value of y.
        - **comperable (Optional bool)**: Determines whether chart can be compared with other charts
        """

        self.chart_name = chart_name
        self.chart_title = chart_title
        self.chart_subtitle = chart_subtitle
        self.chart_type = chart_type
        self.x_data = x_data
        self.y_data = y_data
        self.y_data_names = y_data_names
        self.x_label = x_label
        self.y_label = y_label
        self.x_min = x_min
        self.x_max = x_max
        self.y_min = y_min
        self.y_max = y_max
        self.comparable = comparable

    def get_chart_dictionary(self):
        chart_dictionary = {
            "name": self.chart_name,
            "chart_title": self.chart_title,
            "chart_subtitle": self.chart_subtitle,
            "chart_type": self.chart_type,
            "x_data": self.x_data,
            "y_data": self.y_data,
            "y_data_names": self.y_data_names,
            "x_label": self.x_label,
            "y_label": self.y_label,
            "x_min": self.x_min,
            "x_max": self.x_max,
            "y_min": self.y_min,
            "y_max": self.y_max,
            "comparable": self.comparable
        }

        return chart_dictionary



import os


class Settings:
    """
    Main settings for MLOps library
    """
    def __init__(self):
        self.url: str = "http://127.0.0.1:8000"
        self.active_project_id = None
        self.active_experiment_id = None
        self.active_model: str = None
        self.user_name: str = self.get_username()

        # mailgun variables
        self.mailgun_domain = None
        self.mailgun_api_key = None
        self.user_email = None
        self.send_emails = False

    @staticmethod
    def get_username():
        # Check if the code is running in a GitHub Actions environment
        if os.getenv('GITHUB_ACTIONS') == 'true':
            return os.getenv('GITHUB_ACTOR')
        else:
            return os.getlogin()

    def change_username(self, username: str):
        self.user_name = username

    def change_active_project(self, project_id: str):
        self.active_project_id = project_id

    def change_active_experiment(self, experiment_id: str):
        self.active_experiment_id = experiment_id

    def change_active_model(self, model_name: str):
        self.active_model = model_name

    def set_mailgun_domain(self, mailgun_domain: str):
        self.mailgun_domain = mailgun_domain

    def set_mailgun_api_key(self, mailgun_api_key: str):
        self.mailgun_api_key = mailgun_api_key

    def set_user_email(self, user_email: str):
        self.user_email = user_email

    def set_send_emails_flag(self, send_emails: bool):
        self.send_emails = send_emails


settings = Settings()



from mlops.src.iteration import Iteration


class MonitoredModel:
    """
    Monitored model.

    Attributes:
    - **id (PydanticObjectId)**: Monitored model id.
    - **model_name (str)**: Monitored model name.
    - **model_description (str)**: Monitored model description.
    - **model_status (str)**: Monitored model status.
    - **iteration (Iteration)**: Related Iteration.
    """

    def __init__(self, model_name: str, model_description: str = None, iteration: Iteration = None):
        self.model_name: str = model_name
        self.model_description: str = model_description
        self.iteration: Iteration = iteration



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent.parent

# The text of the README file
README = (HERE/"README.md").read_text()

setup(
   name="mlops-ai",
   version="1.3.0",
   description="Mlops-ai library for managing machine learning projects, experiments, iterations and datasets.",
   long_description=README,
   long_description_content_type="text/markdown",
   URL="https://github.com/kajetsz/mlops/tree/main/library",
   author="Kacper Pękalski, Kajetan Szal, Jędrzej Rybczyński",
   author_email="kac.pekalski1@gmail.com",
   license="Apache License 2.0",
   classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",

   ],
   packages=find_packages(exclude=["tests*"]),
   include_package_data=True,
   install_requires=["requests==2.29.0", "scikit-learn==1.3.0", "torch==2.1.1", "json2html==1.3.0"],
 )
